  PAGE    60,132
  TITLE   pcxTest.ASM -- PCX Programmer's Toolkit 
  SUBTTL  Copyright (c) Genus Microprogramming, Inc. 1988-89

; pcxTest.ASM                                                                ;
; Copyright (c) Genus Microprogramming, Inc. 1988-89  All Rights Reserved.   ;

;****************************************************************************;
;                                                                            ;
; This program merely enters graphics mode, sets the palette, and displays   ;
; the test image "pcxTest.PCX".  It is an example ASM routine for the PCX    ;
; Programmer's Toolkit.                                                      ;
;                                                                            ;
; NOTE: REQUIRES A CGA (or compatible) ADAPTER AND DISPLAY!                  ;
;                                                                            ;
; To Compile: Microsoft ASM 5.x  masm pcxtest;                               ;
;                                link pcxtest,,,pcx_cs;                      ;
;                                                                            ;
; Microsoft ASM 5.x version.              Programmer: Chris Howard  9/08/88  ;
;                                                                            ;
;****************************************************************************;

; Include the PCX Toolkit defines
  INCLUDE pcxlib.asm

; DOS Functions
WRITE     equ       09H                     ;Write a string (term. by EOM)
EXIT      equ       4CH                     ;Exit (TERMINATE)

; String Constants
CR        equ       13                      ;Carriage return
LF        equ       10                      ;Line feed
EOM       equ       '$'                     ;End of Message
ASCIIZ    equ       0                       ;End of string (ASCIIZ format)

; Macros
@DOS      MACRO     func                    ;;Macro to use DOS function calls
          mov       ah,func
          int       21H
          ENDM

@Disp     MACRO     msg                     ;;Macro to display a string
          mov       dx,OFFSET msg           ;; at this location
          @DOS      WRITE
          ENDM

; Start the program (small model)
  .model small

; Set aside some stack space
  .stack 100h

; Program data
  .data

pcxtype   dw        pcxCGA_4
pcximage  db        "pcxTest.PCX",ASCIIZ
pcxpal    db        800 dup(0)

BUFMAX    equ       20000
buffer    db        BUFMAX dup(?)

header    db        CR,LF
          db        "͸",CR,LF
          db        " pcxTest: Example Assembler Test Program    PCX Programmer's Toolkit 3.50 ",CR,LF
          db        " Copyright (c) Genus Microprogramming, Inc. 1988-89  All Rights Reserved. ",CR,LF
          db        ";",CR,LF
          db        CR,LF
          db        "Press a key to run . . ."
          db        EOM

errDisp   db        "Error setting display type",CR,LF,EOM
errMode   db        "Error setting display mode",CR,LF,EOM
errGetPal db        "Error getting palette",     CR,LF,EOM
errSetPal db        "Error setting palette",     CR,LF,EOM
errFile   db        "Error displaying image",    CR,LF,EOM

; Program code
  .code

          EXTRN     pcxSetBuffer            : FAR
          EXTRN     pcxSetDisplay           : FAR
          EXTRN     pcxSetMode              : FAR
          EXTRN     pcxGetFilePalette       : FAR
          EXTRN     pcxSetDisplayPalette    : FAR
          EXTRN     pcxFileDisplay          : FAR

;**********

pcxtest:

; Set up data and stack

          mov       ax,@data                ;Data
          mov       ds,ax

          cli                               ;Stack
          mov       ss,ax
          mov       sp,OFFSET STACK
          sti

; Display program header

          @Disp     header

; Get a key, to begin

          mov       ah,0                    ;Read Key function
          int       16H

; Allocate a larger toolkit buffer, to speed up file and display speed.  
; The toolkit defaults to using an internal 2KB buffer, but if we can    
; spare more memory than that, declare a new buffer.                     
; REMEMBER: This MUST be a GLOBAL buffer (ie, declare OUTSIDE of procs) 

          push      ds                      ;Point to the buffer
          mov       ax,OFFSET buffer
          push      ax
          mov       ax,BUFMAX               ; and say how big it is
          push      ax
          call      pcxSetBuffer            ;Set it

; Set the display type and mode we will be using

          mov       ax,pcxtype              ;Get the pcx image type
          push      ax                      ; and push it on the stack
          call      pcxSetDisplay           ;Try to set the display type

          cmp       ax,pcxSUCCESS           ;Successful?
          je        SETMODE

          @Disp     errDisp                 ;No, so display error message
          jmp       PCXEXIT

SETMODE:

; Now enter graphics mode

          mov       ax,pcxGRAPH             ;Use graphics constant
          push      ax
          call      pcxSetMode              ; and set the mode

          cmp       ax,pcxSUCCESS           ;Successful?
          je        GETPAL

          @Disp     errMode                 ;No, so display error message
          jmp       PCXEXIT

; We are in graphics mode, so get the image file palette
; NOTE: If all your images use a default BIOS palette (or the       
;       default Paintbrush palette), just SKIP setting the palette! 

GETPAL:

          mov       ax,pcxtype              ;Specify the image type
          push      ax
          push      ds                      ; the image name
          mov       ax,OFFSET pcximage
          push      ax
          push      ds                      ; and the palette buffer
          mov       ax,OFFSET pcxpal
          push      ax
          call      pcxGetFilePalette       ; and set the mode

          cmp       ax,pcxSUCCESS           ;Successful?
          je        SETPAL

          @Disp     errGetPal               ;No, so display error message
          jmp       KEY

; Now set it

SETPAL:

          push      ds                      ;Point to the palette location
          mov       ax,OFFSET pcxpal
          push      ax
          call      pcxSetDisplayPalette    ; and set it

          cmp       ax,pcxSUCCESS           ;Successful?
          je        DISPLAY

          @Disp     errSetPal               ;No, so display error message
          jmp       KEY

; Now display the image file

DISPLAY:

          push      ds                      ;Point to the image name
          mov       ax,OFFSET pcximage
          push      ax
          mov       ax,0                    ;Set X, Y, and page to zero
          push      ax                      ; X
          push      ax                      ; Y
          push      ax                      ; Page
          call      pcxFileDisplay          ;Display the file

          cmp       ax,pcxSUCCESS           ;Successful?
          je        KEY

          @Disp     errFile                 ;No, so display error message
;         jmp       KEY

; Wait for a key

KEY:

          mov       ah,0                    ;Read Key function
          int       16H

; Return to text mode

          mov       ax,pcxTEXT              ;Use text constant
          push      ax
          call      pcxSetMode              ; and set the mode

PCXEXIT:

          @DOS      EXIT                    ;Exit back to DOS

          END       pcxtest

